@@ -26,7 +26,9 @@ class Surface:
         for dte in self.df.dte.unique():
             temp = self.df.query("dte == @dte")
             desc = f"{self.symbol} {self.putCall} \n Date: {self.day} \n dte: {dte}"
-            curve = Curve(temp.markIv.values, temp.delta.values, temp.dte, desc)
+            curve = Curve(
+                temp.markIv.values, temp.delta.values, temp.dte, temp.strike, desc
+            )
             self.curves.append(curve)
 
     def getIvFromDeltaAndDte(self, delta, dte):
@@ -50,14 +52,36 @@ class Surface:
                 highestDte - lowestDte
             )
 
+    def getIvFromStrikeAndDte(self, strike, dte):
+        curves = sorted(self.curves, key=lambda x: x.dte)
+        lowestIv = lowestDte = highestIv = highestDte = 0
+        for idx in range(len(curves)):
+            if dte < curves[idx].dte:
+                lowestIv = curves[idx].getIvFromStrike(strike)
+                lowestDte = curves[idx].dte
+                break
+            if dte >= curves[idx].dte:
+                highestIv = curves[idx].getIvFromStrike(strike)
+                highestDte = curves[idx].dte
+
+        if lowestDte == 0:
+            return curves[idx].getIvFromStrike(strike)
+        elif highestDte == 0:
+            return curves[idx - 1].getIvFromStrike(strike)
+        else:
+            return lowestIv + (highestIv - lowestIv) * (dte - lowestDte) / (
+                highestDte - lowestDte
+            )
+
 
 class Curve:
-    def __init__(self, markIvs, deltas, dte, desc):
+    def __init__(self, markIvs, deltas, dte, strikes, desc):
         # Assumes deltas are sorted
         self.markIvs = markIvs
         self.deltas = deltas
         self.dte = dte.min()
         self.desc = desc
+        self.strike = strikes
         assert len(markIvs) == len(deltas), "length mismatch"
 
     def encode_delta(self, delta):
@@ -106,3 +130,26 @@ class Curve:
             return lowestIv + (highestIv - lowestIv) * (delta - lowestDelta) / (
                 highestDelta - lowestDelta
             )
+
+    def getIvFromStrike(self, strike):
+        assert strike > 0
+        lowestIv = lowestStrike = highestIv = highestStrike = 0
+        for idx in range(len(self.str)):
+            if strike < self.strikes[idx]:
+                lowestIv = self.markIvs[idx]
+                lowestStrike = self.strikes[idx]
+                break
+            if strike >= self.strikes[idx]:
+                highestIv = self.markIvs[idx]
+                highestStrike = self.strikes[idx]
+
+        #         print(highestIv, highestDelta, lowestIv, lowestDelta)
+        if lowestStrike == 0:
+            return self.markIvs[idx]
+        elif highestStrike == 0:
+            return self.markIvs[idx - 1]
+        else:
+            #             print(lowestIv + (highestIv-lowestIv)*(delta-lowestDelta)/(highestDelta-lowestDelta))
+            return lowestIv + (highestIv - lowestIv) * (strike - lowestStrike) / (
+                highestStrike - lowestStrike
+            )
